@@ -1,5 +1,20 @@
 # -*- coding: utf-8 -*-
 
+"""
+Wandb is a library to help track machine learning experiments.
+
+For more information on wandb see https://docs.wandb.com.
+
+The most commonly used functions/objects are:
+- wandb.init — initialize a new run at the top of your training script
+- wandb.config — track hyperparameters
+- wandb.log — log metrics over time within your training loop
+- wandb.save — save files in association with your run, like model weights
+- wandb.restore — restore the state of your code when you ran a given run
+
+For examples usage, see https://docs.wandb.com/library/example-projects
+"""
+
 # Three possible modes:
 #     'cli': running from "wandb" command
 #     'run': we're a script launched by "wandb run"
@@ -9,7 +24,7 @@ from __future__ import absolute_import, print_function
 
 __author__ = """Chris Van Pelt"""
 __email__ = 'vanpelt@wandb.com'
-__version__ = '0.8.33'
+__version__ = '0.9.5'
 
 import atexit
 import click
@@ -49,6 +64,7 @@ from wandb import wandb_run
 from wandb import wandb_socket
 from wandb import streaming_log
 from wandb import util
+from wandb.artifacts import Artifact
 from wandb.run_manager import LaunchError, Process
 from wandb.data_types import Image
 from wandb.data_types import Video
@@ -73,6 +89,7 @@ from wandb.wandb_controller import sweep, controller
 
 from wandb.compat import windows
 
+
 logger = logging.getLogger(__name__)
 
 # Internal variables
@@ -111,20 +128,29 @@ def hook_torch(*args, **kwargs):
 _global_watch_idx = 0
 
 
-def watch(models, criterion=None, log="gradients", log_freq=100, idx=None):
+def watch(models, criterion=None, log="gradients", log_freq=1000, idx=None):
     """
-    Hooks into the torch model to collect gradients and the topology.  Should be extended
+    Hooks into a pytorch model to collect gradients and the topology.  Should be extended
     to accept arbitrary ML models.
 
-    :param (torch.Module) models: The model to hook, can be a tuple
-    :param (torch.F) criterion: An optional loss value being optimized
-    :param (str) log: One of "gradients", "parameters", "all", or None
-    :param (int) log_freq: log gradients and parameters every N batches
-    :param (int) idx: an index to be used when calling wandb.watch on multiple models
-    :return: (wandb.Graph) The graph object that will populate after the first backward pass
+    Args:
+        models (torch.Module): The model to hook, can be a tuple
+        criterion (torch.F, optional): An optional loss value being optimized
+        log (str, optional): One of "gradients", "parameters", "all", or None
+            if "gradients" watch will only log gradients
+            if "parameters" watch will only log parameters
+        log_freq (int, optional): log gradients and parameters every N batches
+        idx (int, optional): an index to be used when calling wandb.watch on multiple models
+
+    Returns:
+        :obj:`wandb.Graph` The graph object that will populate after the first backward pass
     """
     global _global_watch_idx
 
+    # TODO: temporary override for huggingface remove after: https://github.com/huggingface/transformers/pull/4220
+    if os.getenv("WANDB_WATCH") == "false":
+        return
+
     if run is None:
         raise ValueError(
             "You must call `wandb.init` before calling watch")
@@ -424,6 +450,7 @@ def _init_jupyter(run):
     """
     from wandb import jupyter
     from IPython.core.display import display, HTML
+
     # TODO: Should we log to jupyter?
     # global logging had to be disabled because it set the level to debug
     # I also disabled run logging because we're rairly using it.
@@ -437,6 +464,8 @@ def _init_jupyter(run):
         # Ensure our api client picks up the new key
         if key:
             run.api.reauth()
+            # Now that we have an api key, let's load the viewer
+            run._load_viewer()
         else:
             run.mode = "dryrun"
             display(HTML('''
@@ -476,16 +505,32 @@ def _init_jupyter(run):
     ipython = get_ipython()
     ipython.register_magics(jupyter.WandBMagics)
 
+    # Monkey patch ipython publish to capture displayed outputs
+    if not hasattr(ipython.display_pub, "_orig_publish"):
+        ipython.display_pub._orig_publish = ipython.display_pub.publish
+    def publish(data, metadata=None, **kwargs):
+        ipython.display_pub._orig_publish(data, metadata=metadata, **kwargs)
+        run._jupyter_agent.save_display(ipython.execution_count , {'data':data, 'metadata':metadata})
+    ipython.display_pub.publish = publish
+
+    # Cell start
     def reset_start():
         """Reset START_TIME to when the cell starts"""
         global START_TIME
         START_TIME = time.time()
+    if hasattr(ipython.events, "_orig_pre_run"):
+        ipython.events.unregister("pre_run_cell", ipython.events._orig_pre_run)
+    ipython.events._orig_pre_run = reset_start
     ipython.events.register("pre_run_cell", reset_start)
 
+    # Cell shutdown
     def cleanup():
         # shutdown async logger because _user_process_finished isn't called in jupyter
         shutdown_async_log_thread()
         run._stop_jupyter_agent()
+    if hasattr(ipython.events, "_orig_post_run"):
+        ipython.events.unregister("post_run_cell", ipython.events._orig_post_run)
+    ipython.events._orig_post_run = cleanup
     ipython.events.register('post_run_cell', cleanup)
 
 
@@ -550,14 +595,63 @@ def join(exit_code=None):
     if len(_global_run_stack) > 0:
         _global_run_stack.pop()
 
+def use_artifact(artifact_or_name, type=None, aliases=None):
+    """ Declare an artifact as an input to a run, call `download` or `file` on \
+        the returned object to get the contents locally.
+
+        Args:
+            artifact_or_name (str or Artifact): An artifact name. May be prefixed with entity/project. Valid names
+                can be in the following forms:
+                    name:version
+                    name:alias
+                    digest
+                You can also pass an Artifact object created by calling `wandb.Artifact`
+            type (str, optional): The type of artifact to use.
+            aliases (list, optional): Aliases to apply to this artifact
+        Returns:
+            A :obj:`Artifact` object.
+    """
+    if run is None:
+        raise ValueError(
+            "You must call `wandb.init` before calling use_artifact")
+    return run.use_artifact(artifact_or_name, type, aliases)
+
+
+def log_artifact(artifact_or_path, name=None, type=None, aliases=['latest']):
+    """ Declare an artifact as output of a run.
+
+        Args:
+            artifact_or_path (str or Artifact): A path to the contents of this artifact,
+                can be in the following forms:
+                    /local/directory
+                    /local/directory/file.txt
+                    s3://bucket/path
+                You can also pass an Artifact object created by calling `wandb.Artifact`
+            name (str, optional): An artifact name. May be prefixed with entity/project. Valid names
+                can be in the following forms:
+                    name:version
+                    name:alias
+                    digest
+                this will default to the basename of the path prepended with the current run id if not specified
+            type (str): The type of artifact to log, examples include "dataset", "model"
+            aliases (list, optional): Aliases to apply to this artifact, defaults to ["latest"]
+        Returns:
+            A :obj:`Artifact` object.
+    """
+    if run is None:
+        raise ValueError(
+            "You must call `wandb.init` before calling log_artifact")
+    return run.log_artifact(artifact_or_path, name, type, aliases)
+
 
 def save(glob_str, base_path=None, policy="live"):
     """ Ensure all files matching *glob_str* are synced to wandb with the policy specified.
 
-    base_path: the base path to run the glob relative to
-    policy:
-        live: upload the file as it changes, overwriting the previous version
-        end: only upload file when the run ends
+    Args:
+        base_path (string): the base path to run the glob relative to
+        policy (string): on of "live" or "end"
+            live: upload the file as it changes, overwriting the previous version
+            end: only upload file when the run ends
     """
     global _saved_files
     if run is None:
@@ -606,15 +700,20 @@ def restore(name, run_path=None, replace=False, root=None):
     """ Downloads the specified file from cloud storage into the current run directory
     if it doesn exist.
 
-    name: the name of the file
-    run_path: optional path to a different run to pull files from
-    replace: whether to download the file even if it already exists locally
-    root: the directory to download the file to.  Defaults to the current
-        directory or the run directory if wandb.init was called.
+    Args:
+        name: the name of the file
+        run_path: optional path to a different run to pull files from
+        replace: whether to download the file even if it already exists locally
+        root: the directory to download the file to.  Defaults to the current
+            directory or the run directory if wandb.init was called.
 
-    returns None if it can't find the file, otherwise a file object open for reading
-    raises wandb.CommError if it can't find the run
+    Returns:
+        None if it can't find the file, otherwise a file object open for reading
+
+    Raises:
+        wandb.CommError if it can't find the run
     """
+
     if run_path is None and run is None:
         raise ValueError(
             "You must call `wandb.init` before calling restore or specify a run_path")
@@ -730,13 +829,94 @@ def shutdown_async_log_thread():
 def log(row=None, commit=None, step=None, sync=True, *args, **kwargs):
     """Log a dict to the global run's history.
 
-    wandb.log({'train-loss': 0.5, 'accuracy': 0.9})
+    wandb.log can be used to log everything from scalars to histograms, media and matplotlib plots.
+
+    The most basic usage is wandb.log({'train-loss': 0.5, 'accuracy': 0.9}). This will save a history
+        row associated with the run with train-loss=0.5 and accuracy=0.9. The history values can be plotted
+        on app.wandb.ai or on a local server. The history values can also be downloaded throught the wandb API.
+
+    Logging a value will update the summary values for any metrics logged. The summary values will
+        appear in the run table at app.wandb.ai or a local server. If a summary value is manually set
+        with for example wandb.run.summary["accuracy"] = 0.9 wandb.log will no longer automatically update
+        the run's accuracy.
+
+    Logging values don't have to be scalars. Logging any wandb object is supported. For example
+        wandb.log({"example": wandb.Image("myimage.jpg")}) will log an example image which will be
+        displayed nicely in the wandb UI. See https://docs.wandb.com/library/reference/data_types
+        for all of the different supported types.
+
+    Logging nested metrics is encouraged and is supported in the wandb API, so you could log multiple accuracy
+        values with wandb.log({'dataset-1-metrics': {'accuracy': 0.9, 'loss': 0.3} ,'dataset-1-metrics': {'accuracy': 0.8, 'loss': 0.2}})
+        and the metrics will be organized in the wandb UI.
+
+    Wandb keeps track of a global step so logging related metrics together is encouraged, so by default each
+        time wandb.log is called a global step is incremented. If it's inconvenient to log related metrics together
+        calling wandb.log({'train-loss': 0.5, commit=False}) and then wandb.log({'accuracy': 0.9}) is equivalent
+        to calling wandb.log({'train-loss': 0.5, 'accuracy': 0.9})
+
+    Wandb.log is not intended to be called more than a few times per second. If you want to log more frequently
+        than that it's better to aggregate the data on the client side or you may get degraded performance.
 
     Args:
         row (dict, optional): A dict of serializable python objects i.e str: ints, floats, Tensors, dicts, or wandb.data_types
-        commit (boolean, optional): Persist a set of metrics, if false just update the existing dict (defaults to true if step is not specified)
-        step (integer, optional): The global step in processing. This persists any non-committed earlier steps but defaults to not committing the specified step
-        sync (boolean, True): If set to False, process calls to log in a seperate thread
+        commit (boolean, optional): Save the metrics dict to the wandb server and increment the step.
+            If false wandb.log just updates the current metrics dict with the row argument and metrics won't
+            be saved until wandb.log is called with commit=True.
+        step (integer, optional): The global step in processing. This persists any non-committed earlier steps but defaults
+            to not committing the specified step.
+        sync (boolean, True): If set to False, process calls to log in a seperate thread.
+
+    Examples:
+        Basic usage
+        ```
+        wandb.log({'accuracy': 0.9, 'epoch': 5})
+        ```
+
+        Incremental logging
+        ```
+        wandb.log({'loss': 0.2}, commit=False)
+        # Somewhere else when I'm ready to report this step:
+        wandb.log({'accuracy': 0.8})
+        ```
+
+        Histogram
+        ```
+        wandb.log({"gradients": wandb.Histogram(numpy_array_or_sequence)})
+        ```
+
+        Image
+        ```
+        wandb.log({"examples": [wandb.Image(numpy_array_or_pil, caption="Label")]})
+        ```
+
+        Video
+        ```
+        wandb.log({"video": wandb.Video(numpy_array_or_path_to_video, fps=4, format="gif")})
+        ```
+
+        Matplotlib Plot
+        ```
+        wandb.log({"chart": plt})
+        ```
+
+        PR Curve
+        ```
+        wandb.log({'pr': wandb.plots.precision_recall(y_test, y_probas, labels)})
+        ```
+
+        3D Object
+        ```
+        wandb.log({"generated_samples":
+           [wandb.Object3D(open("sample.obj")),
+            wandb.Object3D(open("sample.gltf")),
+            wandb.Object3D(open("sample.glb"))]})
+        ```
+
+        For more examples, see https://docs.wandb.com/library/log
+
+    Raises:
+        ValueError - if called before wandb.init
+
     """
 
     if run is None:
@@ -846,7 +1026,7 @@ def sagemaker_auth(overrides={}, path="."):
 def init(job_type=None, dir=None, config=None, project=None, entity=None, reinit=None, tags=None,
          group=None, allow_val_change=False, resume=False, force=False, tensorboard=False,
          sync_tensorboard=False, monitor_gym=False, name=None, notes=None, id=None, magic=None,
-         anonymous=None, config_exclude_keys=None, config_include_keys=None):
+         anonymous=None, config_exclude_keys=None, config_include_keys=None, save_code=None):
     """Initialize W&B
 
     If called from within Jupyter, initializes a new run and waits for a call to
@@ -870,6 +1050,7 @@ def init(job_type=None, dir=None, config=None, project=None, entity=None, reinit
         resume (bool, str, optional): Automatically resume this run if run from the same machine,
             you can also pass a unique run_id
         sync_tensorboard (bool, optional): Synchronize wandb logs to tensorboard or tensorboardX
+        save_code (bool, optional): Save the entrypoint or jupyter session history source code.
         force (bool, optional): Force authentication with wandb, defaults to False
         magic (bool, dict, or str, optional): magic configuration as bool, dict, json string,
             yaml filename
@@ -885,6 +1066,14 @@ def init(job_type=None, dir=None, config=None, project=None, entity=None, reinit
     global __stage_dir__
     global _global_watch_idx
 
+    # TODO: temporary override for huggingface remove after: https://github.com/huggingface/transformers/pull/4220
+    if os.getenv("WANDB_DISABLED") == "true":
+        return None
+    elif wandb_config.huggingface_version() is not None:
+        if InternalApi().api_key is None:
+            termwarn("W&B installed but not logged in.  Run `wandb login` or set the WANDB_API_KEY env variable.")
+            return None
+
     # We allow re-initialization when we're in Jupyter or explicity opt-in to it.
     in_jupyter = _get_python_type() != "python"
     if reinit or (in_jupyter and reinit != False):
@@ -952,7 +1141,8 @@ def init(job_type=None, dir=None, config=None, project=None, entity=None, reinit
             termwarn("Ignoring entity='{}' passed to wandb.init when running a sweep".format(entity))
         if project and project != os.environ.get(env.PROJECT):
             termwarn("Ignoring project='{}' passed to wandb.init when running a sweep".format(project))
-
+    if save_code is not None:
+        os.environ[env.SAVE_CODE]= str(save_code)
     if group:
         os.environ[env.RUN_GROUP] = group
     if job_type:
@@ -964,7 +1154,7 @@ def init(job_type=None, dir=None, config=None, project=None, entity=None, reinit
         os.environ[env.TAGS] = ",".join(tags)
     if id:
         os.environ[env.RUN_ID] = id
-        if name is None and resume is not "must":
+        if name is None and resume != "must":
             # We do this because of https://github.com/wandb/core/issues/2170
             # to ensure that the run's name is explicitly set to match its
             # id. If we don't do this and the id is eight characters long, the
@@ -1042,7 +1232,7 @@ def init(job_type=None, dir=None, config=None, project=None, entity=None, reinit
 
     try:
         signal.signal(signal.SIGQUIT, _debugger)
-    except AttributeError:
+    except (ValueError, AttributeError):
         pass
 
     try:
@@ -1128,6 +1318,7 @@ fastai = util.LazyLoader('fastai', globals(), 'wandb.fastai')
 docker = util.LazyLoader('docker', globals(), 'wandb.docker')
 xgboost = util.LazyLoader('xgboost', globals(), 'wandb.xgboost')
 lightgbm = util.LazyLoader('lightgbm', globals(), 'wandb.lightgbm')
+catboost = util.LazyLoader('catboost', globals(), 'wandb.catboost')
 gym = util.LazyLoader('gym', globals(), 'wandb.gym')
 ray = util.LazyLoader('ray', globals(), 'wandb.ray')
 sklearn = util.LazyLoader('sklearn', globals(), 'wandb.sklearn')
@@ -1135,5 +1326,5 @@ sklearn = util.LazyLoader('sklearn', globals(), 'wandb.sklearn')
 
 __all__ = ['init', 'config', 'summary', 'join', 'login', 'log', 'save', 'restore',
     'tensorflow', 'watch', 'types', 'tensorboard', 'jupyter', 'keras', 'fastai',
-    'docker', 'xgboost', 'gym', 'ray', 'run', 'join', 'Image', 'Video',
+    'docker', 'lightgbm', 'catboost', 'xgboost', 'gym', 'ray', 'run', 'join', 'Image', 'Video',
     'Audio',  'Table', 'Html', 'Object3D', 'Molecule', 'Histogram', 'Graph', 'Api']
