@@ -46,7 +46,9 @@ import socket
 from .core import termlog
 
 import wandb
-from wandb.apis import InternalApi
+from wandb.apis import InternalApi, PublicApi
+from wandb import artifacts
+from wandb.file_pusher import FilePusher
 from wandb.wandb_config import Config
 from wandb.settings import Settings
 from wandb import wandb_agent
@@ -206,7 +208,6 @@ def editor(content='', marker='# Enter a description, markdown is allowed!\n'):
 
 api = InternalApi()
 
-
 # Some commands take project/entity etc. as arguments. We provide default
 # values for those arguments from the current project configuration, as
 # returned by api.settings()
@@ -277,10 +278,9 @@ def runs(ctx, project, entity):
 
 
 @cli.command(context_settings=CONTEXT, help="List local & remote file status")
-@click.argument("run", envvar=env.RUN_ID)
 @click.option("--settings/--no-settings", help="Show the current settings", default=True)
 @display_error
-def status(run, settings):
+def status(settings=None):
     logged_in = bool(api.api_key)
     if not os.path.isdir(wandb_dir()):
         if logged_in:
@@ -446,6 +446,109 @@ def sync(ctx, path, id, project, entity, ignore):
         wandb_run.Run.from_directory(
             path, run_id=id, project=project, entity=entity, ignore_globs=globs)
 
+@cli.group(help="Commands for interacting with artifacts")
+def artifact():
+    pass
+
+@artifact.command(context_settings=CONTEXT, help="Upload an artifact to wandb")
+@click.argument("path")
+@click.option("--name", "-n", help="The name of the artifact to push: project/artifact_name")
+@click.option("--description", "-d", help="A description of this artifact")
+@click.option("--type", "-t", default="dataset", help="The type of the artifact")
+@click.option("--alias", "-a", default=["latest"], multiple=True, help="An alias to apply to this artifact")
+@display_error
+def put(path, name, description, type, alias):
+    if name is None:
+        name = os.path.basename(path)
+    public_api = PublicApi()
+    entity, project, artifact_name = public_api._parse_artifact_path(name)
+    if project is None:
+        project = click.prompt("Enter the name of the project you want to use")
+    # TODO: settings nightmare...
+    api.set_setting("entity", entity)
+    api.set_setting("project", project)
+    artifact = wandb.Artifact(name=artifact_name, type=type, description=description)
+    artifact_path = "{entity}/{project}/{name}:{alias}".format(entity=entity,
+        project=project, name=artifact_name, alias=alias[0])
+    if os.path.isdir(path):
+        wandb.termlog("Uploading directory {path} to: \"{artifact_path}\" ({type})".format(
+            path=path, type=type, artifact_path=artifact_path))
+        artifact.add_dir(path)
+    elif os.path.isfile(path):
+        wandb.termlog("Uploading file {path} to: \"{artifact_path}\" ({type})".format(
+            path=path, type=type, artifact_path=artifact_path))
+        artifact.add_file(path)
+    elif "://" in path:
+        wandb.termlog("Logging reference artifact from {path} to: \"{artifact_path}\" ({type})".format(
+            path=path, type=type, artifact_path=artifact_path))
+        artifact.add_reference(path)
+    else:
+        raise ClickException("Path argument must be a file or directory")
+
+    run = wandb.init(entity=entity, project=project, config={"path": path}, job_type="cli_put")
+    # We create the artifact manually to get the current version
+    res = api.create_artifact(type, artifact_name, artifact.digest,
+        entity_name=entity, project_name=project, run_name=run.id, description=description,
+        aliases=[{"artifactCollectionName": artifact_name, "alias": a} for a in alias])
+    artifact_path = artifact_path.split(":")[0] + ":" + res.get("version", "latest")
+    # Re-create the artifact and actually upload any files needed
+    run.log_artifact(artifact, aliases=alias)
+    wandb.termlog("Artifact uploaded, use this artifact in a run by adding:\n", prefix=False)
+
+    wandb.termlog("    artifact = run.use_artifact(\"{path}\")\n".format(
+        path=artifact_path,
+        type=type
+    ), prefix=False)
+
+
+@artifact.command(context_settings=CONTEXT, help="Download an artifact from wandb")
+@click.argument("path")
+@click.option("--root", help="The directory you want to download the artifact to")
+@click.option("--type", help="The type of artifact you are downloading")
+@display_error
+def get(path, root, type):
+    public_api = PublicApi()
+    entity, project, artifact_name = public_api._parse_artifact_path(path)
+    if project is None:
+        project = click.prompt("Enter the name of the project you want to use")
+
+    try:
+        artifact_parts = artifact_name.split(":")
+        if len(artifact_parts) > 1:
+            version = artifact_parts[1]
+            artifact_name = artifact_parts[0]
+        else:
+            version = "latest"
+        full_path = "{entity}/{project}/{artifact}:{version}".format(
+            entity=entity, project=project,
+            artifact=artifact_name, version=version)
+        wandb.termlog("Downloading {type} artifact {full_path}".format(
+            type=type or "dataset", full_path=full_path))
+        artifact = public_api.artifact(full_path, type=type)
+        path = artifact.download(root=root)
+        wandb.termlog("Artifact downloaded to %s" % path)
+    except ValueError:
+        raise ClickException("Unable to download artifact")
+
+@artifact.command(context_settings=CONTEXT, help="List all artifacts in a wandb project")
+@click.argument("path")
+@click.option("--type", "-t", help="The type of artifacts to list")
+@display_error
+def ls(path, type):
+    public_api = PublicApi()
+    if type is not None:
+        types = [public_api.artifact_type(type, path)]
+    else:
+        types = public_api.artifact_types(path)
+
+    def human_size(bytes, units=['','KB','MB','GB','TB', 'PB', 'EB']):
+        return str(bytes) + units[0] if bytes < 1024 else human_size(bytes>>10, units[1:])
+
+    for kind in types:
+        for collection in kind.collections():
+            versions = public_api.artifact_versions(kind.type, "/".join([kind.entity, kind.project, collection.name]), per_page=1)
+            latest = next(versions)
+            print("{:<15s}{:<15s}{:>15s} {:<20s}".format(kind.type, latest.updated_at, human_size(latest.size), latest.name))
 
 @cli.command(context_settings=CONTEXT, help="Pull files from Weights & Biases")
 @click.argument("run", envvar=env.RUN_ID)
@@ -454,7 +557,6 @@ def sync(ctx, path, id, project, entity, ignore):
 @display_error
 def pull(run, project, entity):
     project, run = api.parse_slug(run, project=project)
-
     urls = api.download_urls(project, run=run, entity=entity)
     if len(urls) == 0:
         raise ClickException("Run has no files")
