@@ -140,7 +140,8 @@ class Media(WBValue):
         if extension is not None and not path.endswith(extension):
             raise ValueError('Media file extension "{}" must occur at the end of path "{}".'.format(extension, path))
 
-        self._sha256 = hashlib.sha256(open(self._path, 'rb').read()).hexdigest()
+        with open(self._path, 'rb') as f:
+            self._sha256 = hashlib.sha256(f.read()).hexdigest()
         self._size = os.path.getsize(self._path)
 
     @classmethod
@@ -247,15 +248,23 @@ class Table(Media):
         columns ([str]): Names of the columns in the table.
             Defaults to ["Input", "Output", "Expected"].
         data (array): 2D Array of values that will be displayed as strings.
+        dataframe (pandas.DataFrame): DataFrame object used to create the table.
+            When set, the other arguments are ignored.
     """
     MAX_ROWS = 10000
 
-    def __init__(self, columns=["Input", "Output", "Expected"], data=None, rows=None):
+    def __init__(self, columns=["Input", "Output", "Expected"], data=None, rows=None, dataframe=None):
         """rows is kept for legacy reasons, we use data to mimic the Pandas api
         """
         super(Table, self).__init__()
         self.columns = columns
         self.data = list(rows or data or [])
+        if dataframe is not None:
+            assert util.is_pandas_data_frame(dataframe), 'dataframe argument expects a `Dataframe` object'
+            self.columns = list(dataframe.columns)
+            self.data = []
+            for row in range(len(dataframe)):
+                self.add_data(*tuple(dataframe[col].values[row] for col in self.columns))
 
     def add_row(self, *row):
         logging.warning("add_row is deprecated, use add_data")
@@ -456,6 +465,7 @@ class Object3D(BatchableMedia):
             if data_or_path['type'] == 'lidar/beta':
                 data = {
                     'type': data_or_path['type'],
+                    'vectors': data_or_path['vectors'].tolist(),
                     'points': data_or_path['points'].tolist(),
                     'boxes': data_or_path['boxes'].tolist(),
                 }
@@ -861,13 +871,15 @@ class Image(BatchableMedia):
                 masks_final[key] = ImageMask(masks[key], key)
             self._masks = masks_final
 
+        PILImage = util.get_module(
+            "PIL.Image", required='wandb.Image needs the PIL package. To get it, run "pip install pillow".')
+
         if isinstance(data_or_path, six.string_types):
             self._set_file(data_or_path, is_tmp=False)
+            self._image = PILImage.open(data_or_path)
         else:
             data = data_or_path
 
-            PILImage = util.get_module(
-                "PIL.Image", required='wandb.Image needs the PIL package. To get it, run "pip install pillow".')
             if util.is_matplotlib_typename(util.get_full_typename(data)):
                 buf = six.BytesIO()
                 util.ensure_matplotlib_figure(data).savefig(buf)
@@ -890,12 +902,13 @@ class Image(BatchableMedia):
                 self._image = PILImage.fromarray(
                     self.to_uint8(data), mode=mode or self.guess_mode(data))
 
-            self._width, self._height = self._image.size
 
             tmp_path = os.path.join(MEDIA_TMP.name, util.generate_id() + '.png')
             self._image.save(tmp_path, transparency=None)
             self._set_file(tmp_path, is_tmp=True)
 
+        self._width, self._height = self._image.size
+
     @classmethod
     def get_media_subdir(cls):
         return os.path.join('media', 'images')
@@ -1196,9 +1209,9 @@ class ImageMask(Media):
     def __init__(self, val, key, **kwargs):
         super(ImageMask, self).__init__()
 
+        np = util.get_module("numpy", required="Semantic Segmentation mask support requires numpy")
         # Add default class mapping
         if not "class_labels" in val:
-            np = util.get_module("numpy", required="Semantic Segmentation mask support requires numpy")
             classes = np.unique(val["mask_data"]).astype(np.int32).tolist()
             class_labels = dict((c, "class_" + str(c)) for c in classes)
             val["class_labels"] = class_labels
@@ -1212,7 +1225,7 @@ class ImageMask(Media):
 
         PILImage = util.get_module(
             "PIL.Image", required='wandb.Image needs the PIL package. To get it, run "pip install pillow".')
-        image = PILImage.fromarray(val["mask_data"], mode="L")
+        image = PILImage.fromarray(val["mask_data"].astype(np.int8), mode="L")
 
         image.save(tmp_path, transparency=None)
         self._set_file(tmp_path, is_tmp=True, extension=ext)
@@ -1238,6 +1251,8 @@ class ImageMask(Media):
         return "mask"
 
     def validate(self, mask):
+        np = util.get_module("numpy", required="Semantic Segmentation mask support requires numpy")
+
         # 2D Make this work with all tensor(like) types
         if not "mask_data" in mask:
             raise TypeError("Missing key \"mask_data\": A mask requires mask data(A 2D array representing the predctions)")
@@ -1246,8 +1261,7 @@ class ImageMask(Media):
             shape = mask["mask_data"].shape
             if len(shape) != 2:
                 raise TypeError(error_str)
-            if not ((mask["mask_data"] >= 0).all() and (mask["mask_data"] <= 255).all()) and \
-                    mask["mask_data"].dtype('int8').kind == "i":
+            if not ((mask["mask_data"] >= 0).all() and (mask["mask_data"] <= 255).all()) and issubclass(mask["mask_data"].dtype.type, np.integer):
                 raise TypeError("Mask data must be integers between 0 and 255")
 
         # Optional argument
@@ -1271,7 +1285,6 @@ class Plotly(Media):
             val = util.ensure_matplotlib_figure(val)
             if any(len(ax.images) > 0 for ax in val.axes):
                 return Image(val)
-            val = util.ensure_matplotlib_figure(val)
             # otherwise, convert to plotly figure
             tools = util.get_module(
                 "plotly.tools", required="plotly is required to log interactive plots, install with: pip install plotly or convert the plot to an image with `wandb.Image(plt)`")
@@ -1721,7 +1734,7 @@ def val_to_json(run, key, val, step='summary'):
     typename = util.get_full_typename(val)
 
     if util.is_pandas_data_frame(val):
-        assert step == 'summary', "We don't yet support DataFrames in History."
+        assert step == 'summary', "We don't yet support DataFrames in History. Use `wandb.Table(dataframe=my_dataframe)`"
         return data_frame_to_json(val, run, key, step)
     elif util.is_matplotlib_typename(typename) or util.is_plotly_typename(typename):
         val = Plotly.make_plot_media(val)
