@@ -34,6 +34,7 @@ from wandb.apis import file_stream
 from wandb import __version__
 from wandb import env as wandb_env
 from wandb import Error
+from wandb import artifacts
 from wandb import io_wrap
 from wandb import jsonlfile
 from wandb import file_pusher
@@ -278,7 +279,8 @@ class FileEventHandlerConfig(FileEventHandler):
 
     def finish(self):
         if self._thread:
-            self._thread.join()
+            # Cancel the current thread to keep moving
+            self._thread.cancel()
             self._thread = None
 
         self._update()
@@ -664,11 +666,9 @@ class RunManager(object):
         """Stops file syncing/streaming but doesn't actually wait for everything to
         finish. We print progress info later.
         """
-
         # TODO: there was a case where _file_event_handlers was getting modified in the loop.
         for handler in list(self._file_event_handlers.values()):
             handler.finish()
-
         self._file_pusher.finish()
         self._api.get_file_stream_api().finish(exitcode)
         # In Jupyter notebooks, wandb.init can be called multiple times in the same
@@ -705,11 +705,11 @@ class RunManager(object):
         new_save_name = os.path.relpath(event.dest_path, self._run.dir)
         self._ensure_file_observer_is_unblocked()
 
-        # We have to move the existing file handler to the new name, and update the stats
+        # We have to move the existing file handler to the new name. The old file
+        # path is still sync'd as well (we don't try to delete the old path).
         handler = self._get_file_event_handler(event.src_path, old_save_name)
         self._file_event_handlers[new_save_name] = handler
         del self._file_event_handlers[old_save_name]
-        self._file_pusher.rename_file(old_save_name, new_save_name, event.dest_path)
 
         handler.on_renamed(event.dest_path, new_save_name)
 
@@ -719,8 +719,6 @@ class RunManager(object):
         file_path: the file's actual path
         save_name: its path relative to the run directory (aka the watch directory)
         """
-        self._file_pusher.update_file(save_name, file_path)  # track upload progress
-
         if save_name not in self._file_event_handlers:
             if save_name == 'wandb-history.jsonl':
                 self._api.get_file_stream_api().set_file_policy(save_name, file_stream.JsonlFilePolicy())
@@ -789,10 +787,12 @@ class RunManager(object):
             fs_api, util.OUTPUT_FNAME, prepend_timestamp=True, line_prepend='ERROR'))
 
     def unmirror_stdout_stderr(self):
-        # Python 2 tests were failing...
-        if hasattr(sys.stdout, "orig_write"):
+        try:
             sys.stdout.write = sys.stdout.orig_write
             sys.stderr.write = sys.stderr.orig_write
+        # Python 2 tests sometimes failed...
+        except AttributeError:
+            pass
 
     def _get_stdout_stderr_streams(self):
         """Sets up STDOUT and STDERR streams. Only call this once."""
@@ -1048,7 +1048,7 @@ class RunManager(object):
 
         env = self._run.set_environment(environment=env)
 
-        if not env.get(wandb_env.DISABLE_CODE):
+        if wandb_env.should_save_code():
             logger.info("saving patches")
             self._api.save_patches(self._run.dir)
         if env.get("SPELL_RUN_URL"):
@@ -1080,11 +1080,12 @@ class RunManager(object):
         if self._run_status_checker:
             self._run_status_checker.shutdown()
 
+        self._run.history.close()
+
         if self._cloud:
             logger.info("stopping streaming files and file change observer")
             self._end_file_syncing(exitcode)
 
-        self._run.history.close()
 
     def run_user_process(self, program, args, env):
         """Launch a user process, capture its output, and sync its files to the backend.
@@ -1195,6 +1196,31 @@ class RunManager(object):
                 if current and policy["policy"] == "live" and not is_live:
                     del self._file_event_handlers[save_name]
             self._user_file_policies[policy["policy"]].append(policy["glob"])
+    
+    def use_artifact(self, message):
+        type = message['type']
+        name = message['name']
+        server_manifest_entries = message['server_manifest_entries']
+        digest = message['digest']
+        metadata = message['metadata']
+        manifest = message['manifest']
+        aliases = message['aliases']
+        la = artifacts.ArtifactSaver(self._api, digest, server_manifest_entries, manifest, file_pusher=self._file_pusher, is_user_created=True)
+        server_artifact = la.save(type, name, metadata=metadata, aliases=aliases, use_after_commit=True)
+
+    def log_artifact(self, message):
+        type = message['type']
+        name = message['name']
+        server_manifest_entries = message['server_manifest_entries']
+        manifest = message['manifest']
+        description = message['description']
+        digest = message['digest']
+        labels = message['labels'] if 'labels' in message else None
+        metadata = message['metadata'] if 'metadata' in message else None
+        aliases = message['aliases'] if 'aliases' in message else None
+
+        la = artifacts.ArtifactSaver(self._api, digest, server_manifest_entries, manifest, file_pusher=self._file_pusher)
+        la.save(type, name, description=description, metadata=metadata, aliases=aliases, labels=labels)
 
     def start_tensorboard_watcher(self, logdir, save=True):
         try:
@@ -1278,7 +1304,7 @@ class RunManager(object):
                     payload = b''
                 else:
                     try:
-                        res = self._socket.recv(1024)
+                        res = self._socket.recv(64 * 1024)
                     except socket.error as e:
                         # https://stackoverflow.com/questions/16094618/python-socket-recv-and-signals
                         if e.errno == errno.EINTR or isinstance(e, socket.timeout):
@@ -1302,19 +1328,21 @@ class RunManager(object):
                         break
                     elif parsed.get("save_policy"):
                         self.update_user_file_policy(parsed["save_policy"])
-                        payload = b''
-                        parse = False
                     elif parsed.get("tensorboard"):
                         if parsed["tensorboard"].get("logdir"):
                             self.start_tensorboard_watcher(parsed["tensorboard"]["logdir"], parsed["tensorboard"]["save"])
-                        payload = b''
-                        parse = False
+                    elif parsed.get("use_artifact"):
+                        self.use_artifact(parsed["use_artifact"])
+                    elif parsed.get("log_artifact"):
+                        self.log_artifact(parsed["log_artifact"])
                     else:
                         message = "Invalid message received from child process: %s" % str(
                             payload)
                         wandb.termerror(message)
                         util.sentry_exc(message)
                         break
+                    payload = b''
+                    parse = False
                     new_start = term + 1
                     # There's more to parse, add the remaining bytes
                     if len(res) > new_start:
@@ -1323,7 +1351,6 @@ class RunManager(object):
                     exitcode = self.proc.poll()
                     if exitcode is not None:
                         break
-                    time.sleep(1)
         except KeyboardInterrupt:
             logger.info("process received interrupt signal, shutting down")
             exitcode = 255
@@ -1427,19 +1454,12 @@ class RunManager(object):
                 format_str = u'  {:>%s} {}' % max_len
                 wandb.termlog(format_str.format(key, line))
 
-        wandb_files = set([save_name for save_name in self._file_pusher.files() if util.is_wandb_file(save_name)])
-        media_files = set([save_name for save_name in self._file_pusher.files() if save_name.startswith('media')])
-        other_files = set(self._file_pusher.files()) - wandb_files - media_files
+        file_counts = self._file_pusher.file_counts_by_category()
+
         logger.info("syncing files to cloud storage")
-        if other_files:
-            wandb.termlog('Syncing files in %s:' % os.path.relpath(self._run.dir))
-            for save_name in sorted(other_files):
-                wandb.termlog('  %s' % save_name)
-            wandb.termlog('plus {} W&B file(s) and {} media file(s)'.format(len(wandb_files), len(media_files)))
-        else:
-            wandb.termlog('Syncing {} W&B file(s) and {} media file(s)'.format(len(wandb_files), len(media_files)))
+        wandb.termlog('Syncing {} W&B file(s), {} media file(s), {} artifact file(s) and {} other file(s)'.format(
+            file_counts['wandb'], file_counts['media'], file_counts['artifact'], file_counts['other']))
 
-        self._file_pusher.update_all_files()
         self._file_pusher.print_status()
 
         try:
