@@ -7,7 +7,6 @@ import numpy as np
 import scipy as sp
 from wandb.sklearn.utils import *
 from sklearn.base import clone
-from joblib import Parallel, delayed
 from sklearn import model_selection
 from sklearn import datasets
 from sklearn import metrics
@@ -197,7 +196,6 @@ def summary_metrics(model=None, X=None, y=None, X_test=None, y_test=None):
         # Classifier Metrics
         if sklearn.base.is_classifier(model):
             y_pred = model.predict(X_test)
-            y_probas = model.predict_proba(X_test)
 
             metric_name.append("accuracy_score")
             metric_value.append(round_2(sklearn.metrics.accuracy_score(y_test, y_pred)))
@@ -370,7 +368,8 @@ def confusion_matrix(y_true=None, y_pred=None, labels=None, true_labels=None,
         if true_labels is None:
             true_classes = classes
         else:
-            validate_labels(classes, true_labels, "true_labels")
+            # TODO: Where did this go?
+            # validate_labels(classes, true_labels, "true_labels")
 
             true_label_indexes = np.in1d(classes, true_labels)
 
@@ -380,7 +379,8 @@ def confusion_matrix(y_true=None, y_pred=None, labels=None, true_labels=None,
         if pred_labels is None:
             pred_classes = classes
         else:
-            validate_labels(classes, pred_labels, "pred_labels")
+            # TODO: Where did this go?
+            # validate_labels(classes, pred_labels, "pred_labels")
 
             pred_label_indexes = np.in1d(classes, pred_labels)
 
@@ -482,17 +482,39 @@ def plot_feature_importances(model=None, feature_names=None,
     Example:
         wandb.sklearn.plot_feature_importances(model, ['width', 'height, 'length'])
     """
-    if not hasattr(model, 'feature_importances_'):
-        wandb.termwarn("feature_importances_ attribute not in classifier. Cannot plot feature importances.")
+    attributes_to_check = ['feature_importances_', 'coef_']
+    def get_attributes_as_formatted_string():
+        result = ''
+        for index in range(len(attributes_to_check) - 1):
+            if result == '':
+                result = attributes_to_check[index]
+            else:
+                result = ", ".join([result, attributes_to_check[index]])
+
+        return " or ".join([result, attributes_to_check[-1]])
+
+    def check_for_attribute_on(model):
+        for each in attributes_to_check:
+            if hasattr(model, each):
+                return each
+        return None
+
+    found_attribute = check_for_attribute_on(model)
+    if found_attribute is None:
+        wandb.termwarn("%s attribute not in classifier. Cannot plot feature importances." % get_attributes_as_formatted_string())
         return
+
     if (test_missing(model=model) and test_types(model=model) and
         test_fitted(model)):
         feature_names = np.asarray(feature_names)
-        importances = model.feature_importances_
+        if found_attribute == 'feature_importances_':
+            importances = model.feature_importances_
+        if found_attribute == 'coef_':  # ElasticNet or ElasticNetCV like models
+            importances = model.coef_
 
         indices = np.argsort(importances)[::-1]
 
-        if feature_names is None:
+        if feature_names is None or not feature_names.astype(np.object).any():
             feature_names = indices
         else:
             feature_names = np.array(feature_names)[indices]
@@ -512,8 +534,7 @@ def plot_feature_importances(model=None, feature_names=None,
                 ]
             ))
         wandb.log({'feature_importances': feature_importances_table(feature_names, importances)})
-        return
-
+        return feature_importances_table(feature_names, importances)
 
 def plot_elbow_curve(clusterer=None, X=None, cluster_ranges=None, n_jobs=1,
                     show_cluster_time=True):
@@ -536,6 +557,11 @@ def plot_elbow_curve(clusterer=None, X=None, cluster_ranges=None, n_jobs=1,
     Example:
         wandb.sklearn.plot_elbow_curve(model, X_train)
     """
+    try:
+        from joblib import Parallel, delayed
+    except ImportError:
+        wandb.termerror("plot_elbow_curve requires python 3x")
+        return
     if not hasattr(clusterer, 'n_clusters'):
         wandb.termlog('n_clusters attribute not in classifier. Cannot plot elbow method.')
         return
