@@ -14,6 +14,7 @@ from sentry_sdk import configure_scope
 
 from . import env
 import wandb
+from wandb import artifacts
 from wandb import history
 from wandb import jsonlfile
 from wandb import summary
@@ -23,8 +24,9 @@ from wandb import util
 from wandb.core import termlog
 from wandb import data_types
 from wandb.file_pusher import FilePusher
-from wandb.apis import InternalApi, CommError
-from wandb.wandb_config import Config, ConfigStatic
+from wandb.apis import InternalApi, PublicApi, CommError
+from wandb.apis.public import Artifact as ApiArtifact
+from wandb.wandb_config import Config, ConfigStatic, is_kaggle
 from wandb.viz import Visualize
 import six
 from six.moves import input
@@ -44,6 +46,17 @@ DESCRIPTION_FNAME = 'description.md'
 
 
 class Run(object):
+    """
+    Run is a wandb run which corresponds to a machine learning experiment on one machine.
+
+    Example:
+        ```
+        wandb.init(project="runs-from-for-loop", reinit=True)
+        for y in range (100):
+            run.log({"metric": y})
+        ```
+    """
+
     def __init__(self, run_id=None, mode=None, dir=None, group=None, job_type=None,
                  config=None, sweep_id=None, storage_id=None, description=None, resume=None,
                  program=None, args=None, wandb_dir=None, tags=None, name=None, notes=None,
@@ -149,6 +162,9 @@ class Run(object):
         self._meta = None
         self._run_manager = None
         self._jupyter_agent = None
+        self._viewer = None
+        self._flags = {}
+        self._load_viewer()
 
         # give access to watch method
         self.watch = wandb.watch
@@ -177,6 +193,18 @@ class Run(object):
         # TODO: theres an edge case where self.entity is None
         return "/".join([str(self.entity), self.project_name(), self.id])
 
+    def _load_viewer(self):
+        if self.mode != "dryrun" and not self._api.disabled() and self._api.api_key:
+            # Kaggle has internet disabled by default, this checks for that case
+            async_viewer = util.async_call(self._api.viewer, timeout=env.get_http_timeout(5))
+            viewer, viewer_thread = async_viewer()
+            if viewer_thread.is_alive():
+                if is_kaggle():
+                    raise CommError("To use W&B in kaggle you must enable internet in the settings panel on the right.")
+            else:
+                self._viewer = viewer
+                self._flags = json.loads(viewer.get("flags", "{}"))
+
     def _init_jupyter_agent(self):
         from wandb.jupyter import JupyterAgent
         self._jupyter_agent = JupyterAgent()
@@ -188,9 +216,9 @@ class Run(object):
         """ Sends a message to the wandb process changing the policy
         of saved files.  This is primarily used internally by wandb.save
         """
-        if not options.get("save_policy") and not options.get("tensorboard"):
+        if not options.get("save_policy") and not options.get("tensorboard") and not options.get("log_artifact") and not options.get("use_artifact"):
             raise ValueError(
-                "Only configuring save_policy and tensorboard is supported")
+                "Only configuring save_policy, tensorboard, use_artifact and log_artifact is supported")
         if self.socket:
             # In the user process
             self.socket.send(options)
@@ -203,6 +231,10 @@ class Run(object):
             elif options.get("tensorboard"):
                 self._jupyter_agent.rm.start_tensorboard_watcher(
                     options["tensorboard"]["logdir"], options["tensorboard"]["save"])
+            elif options.get("use_artifact"):
+                self._jupyter_agent.rm.use_artifact(options["use_artifact"])
+            elif options.get("log_artifact"):
+                self._jupyter_agent.rm.log_artifact(options["log_artifact"])
         elif self._run_manager:
             # Running in the wandb process, used for tfevents saving
             if options.get("save_policy"):
@@ -318,10 +350,13 @@ class Run(object):
             from wandb import tensorflow as wbtf
             wandb.termlog("Found tfevents file, converting...")
             summary = {}
+            step = 0
             for path in tfevents:
                 filename = os.path.basename(path)
                 namespace = path.replace(filename, "").replace(directory, "").strip(os.sep)
-                summary.update(wbtf.stream_tfevents(path, file_api, run, namespace=namespace))
+                last_row = wbtf.stream_tfevents(path, file_api, run, step=step, namespace=namespace)
+                step = last_row["_step"]
+                summary.update(last_row)
             for path in glob.glob(os.path.join(directory, "media/**/*"), recursive=True):
                 if os.path.isfile(path):
                     paths.append(path)
@@ -360,7 +395,6 @@ class Run(object):
         pusher = FilePusher(api)
         for k in paths:
             path = os.path.abspath(os.path.join(directory, k))
-            pusher.update_file(k, path)
             pusher.file_changed(k, path)
         pusher.finish()
         pusher.print_status()
@@ -391,6 +425,120 @@ class Run(object):
         self.name = upsert_result.get('displayName')
         return upsert_result
 
+    def use_artifact(self, artifact, type=None, aliases=None):
+        """ Declare an artifact as an input to a run, call `download` or `file` on \
+        the returned object to get the contents locally.
+
+        Args:
+            artifact_or_name (str or Artifact): An artifact name. May be prefixed with entity/project. Valid names
+                can be in the following forms:
+                    name:version
+                    name:alias
+                    digest
+                You can also pass an Artifact object created by calling `wandb.Artifact`
+            type (str, optional): The type of artifact to use.
+            aliases (list, optional): Aliases to apply to this artifact
+        Returns:
+            A :obj:`Artifact` object.
+        """
+        if self.mode == "dryrun":
+            wandb.termwarn("Using artifacts in dryrun mode is currently unsupported.")
+            return artifact
+        self.history.ensure_jupyter_started()
+        if isinstance(artifact, six.string_types):
+            # Ensure we have an entity (_load_entity puts the result into api.settings())
+            # then initialize PublicApi with it.
+            self._load_entity(self.api, True)
+            public_api = PublicApi(self.api.settings())
+            artifact = public_api.artifact(name=artifact)
+            if type is not None and type != artifact.type:
+                raise ValueError('Supplied type {} does not match type {} of artifact {}'.format(
+                    type, artifact.type, artifact.name))
+            self.api.use_artifact(artifact.id, entity_name=self.entity, project_name=self.project_name())
+            return artifact
+        else:
+            if isinstance(aliases, six.string_types):
+                aliases = [aliases]
+            if isinstance(artifact, wandb.Artifact):
+                artifact.finalize()
+                self.send_message({
+                    'use_artifact': {
+                        'type': artifact.type,
+                        'name': artifact.name,
+                        'server_manifest_entries': artifact.server_manifest.entries,
+                        'manifest': artifact.manifest.to_manifest_json(include_local=True),
+                        'digest': artifact.digest,
+                        'metadata': artifact.metadata,
+                        'aliases': aliases
+                    }
+                })
+                return artifact
+            elif isinstance(artifact, ApiArtifact):
+                self.api.use_artifact(artifact.id)
+                return artifact
+            else:
+                raise ValueError('You must pass an artifact name (e.g. "pedestrian-dataset:v1"), an instance of wandb.Artifact, or wandb.Api().artifact() to use_artifact')
+
+    def log_artifact(self, artifact_or_path, name=None, type=None, aliases=['latest']):
+        """ Declare an artifact as output of a run.
+
+        Args:
+            artifact_or_path (str or Artifact): A path to the contents of this artifact,
+                can be in the following forms:
+                    /local/directory
+                    /local/directory/file.txt
+                    s3://bucket/path
+                You can also pass an Artifact object created by calling `wandb.Artifact`
+            name (str, optional): An artifact name. May be prefixed with entity/project. Valid names
+                can be in the following forms:
+                    name:version
+                    name:alias
+                    digest
+                this will default to the basename of the path prepended with the current run id  if not specified
+            type (str): The type of artifact to log, examples include "dataset", "model"
+            aliases (list, optional): Aliases to apply to this artifact, defaults to ["latest"]
+        Returns:
+            A :obj:`Artifact` object.
+        """
+        if isinstance(artifact_or_path, six.string_types):
+            if name is None:
+                name = "run-%s-%s" % (self.id, os.path.basename(artifact_or_path))
+            artifact = wandb.Artifact(name, type)
+            if os.path.isfile(artifact_or_path):
+                artifact.add_file(artifact_or_path)
+            elif os.path.isdir(artifact_or_path):
+                artifact.add_dir(artifact_or_path)
+            elif "://" in artifact_or_path:
+                artifact.add_reference(artifact_or_path)
+            else:
+                raise ValueError("path must be a file, directory or external reference like s3://bucket/path")
+        else:
+            artifact = artifact_or_path
+        if not isinstance(artifact, artifacts.Artifact):
+            raise ValueError('You must pass an instance of wandb.Artifact or a valid file path to log_artifact')
+        if self.mode == "dryrun":
+            wandb.termwarn("Persisting artifacts in dryrun mode is currently unsupported.")
+            return artifact
+        self.history.ensure_jupyter_started()
+        if isinstance(aliases, str):
+            aliases = [aliases]
+
+        artifact.finalize()
+        self.send_message({
+            'log_artifact': {
+                'type': artifact.type,
+                'name': artifact.name,
+                'server_manifest_entries': artifact.server_manifest.entries,
+                'manifest': artifact.manifest.to_manifest_json(include_local=True),
+                'digest': artifact.digest,
+                'description': artifact.description,
+                'metadata': artifact.metadata,
+                'labels': None,
+                'aliases': aliases,
+            }
+        })
+        return artifact
+
     def set_environment(self, environment=None):
         """Set environment variables needed to reconstruct this object inside
         a user scripts (eg. in `wandb.init()`).
@@ -404,6 +552,12 @@ class Run(object):
         environment[env.MODE] = self.mode
         environment[env.RUN_DIR] = self.dir
 
+        # Load global environment vars from viewer flags
+        # This should be scoped to entity / project, this work is happening in CLI-NG
+        if self._flags.get("code_saving_enabled") is not None:
+            if environment.get(env.SAVE_CODE) is None:
+                environment[env.SAVE_CODE] = str(self._flags["code_saving_enabled"])
+
         if self.group:
             environment[env.RUN_GROUP] = self.group
         if self.job_type:
@@ -452,10 +606,11 @@ class Run(object):
         entity = api.settings('entity')
         if network:
             if api.settings('entity') is None:
-                viewer = api.viewer()
-                if viewer.get('entity'):
-                    api.set_setting('entity', viewer['entity'])
-
+                if self._viewer:
+                    if self._viewer.get('entity'):
+                        api.set_setting('entity', self._viewer['entity'])
+                    else:
+                        raise CommError("Can't connect to network to query viewer from API key")
             entity = api.settings('entity')
 
         if not entity:
@@ -522,7 +677,6 @@ class Run(object):
             query_string=self._generate_query_string(api, params)
         )
 
-
     def upload_debug(self):
         """Uploads the debug log to cloud storage"""
         if os.path.exists(self.log_fname):
@@ -617,6 +771,11 @@ class Run(object):
 
     @property
     def summary(self):
+        """
+        The runs summary or output metrics from the run.
+
+        Calling wandb.log will set the corresponding summary metric if run.summary has not been set directly.
+        """
         if self._summary is None:
             self._summary = summary.FileSummary(self)
         return self._summary
@@ -629,6 +788,20 @@ class Run(object):
         self.summary.update(row, overwrite=False)
 
     def log(self, row=None, commit=None, step=None, sync=True, *args, **kwargs):
+        """
+        Logs a dict to the run's history.
+
+        For more details see wandb.log()
+
+        Args:
+            row (dict, optional): A dict of serializable python objects i.e str: ints, floats, Tensors, dicts, or wandb.data_types
+            commit (boolean, optional): Save the metrics dict to the wandb server and increment the step.
+                If false wandb.log just updates the current metrics dict with the row argument and metrics won't
+                be saved until wandb.log is called with commit=True.
+            step (integer, optional): The global step in processing. This persists any non-committed earlier steps but defaults
+                to not committing the specified step.
+            sync (boolean, True): If set to False, process calls to log in a seperate thread.
+        """
         if sync == False:
             wandb._ensure_async_log_thread_started()
             return wandb._async_log_queue.put({"row": row, "commit": commit, "step": step})
@@ -747,10 +920,11 @@ class Run(object):
         wandb.join(exit_code)
         return exc_type is None
 
+
 def run_dir_path(run_id, dry=False):
     if dry:
         prefix = 'dryrun'
     else:
         prefix = 'run'
-    time_str = datetime.datetime.utcnow().strftime('%Y%m%d_%H%M%S')
+    time_str = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
     return os.path.join(wandb.wandb_dir(), '{}-{}-{}'.format(prefix, time_str, run_id))
