@@ -162,7 +162,7 @@ class TorchHistory(object):
             sparse_zeros = all_values - non_zero_values
             tensor = backing_values
 
-        flat = tensor.view(-1)
+        flat = tensor.contiguous().view(-1)
 
         # For pytorch 0.3 we use unoptimized numpy histograms (detach is new in 0.4)
         if not hasattr(flat, "detach"):
@@ -210,6 +210,10 @@ class TorchHistory(object):
             # If we've got zeros to add in, make sure zero is in the hist range.
             tmin = 0 if tmin > 0 else tmin
             tmax = 0 if tmax < 0 else tmax
+        # Anecdotally, this can somehow happen sometimes. Maybe a precision error
+        # in min()/max() above. Swap here to prevent a runtime error.
+        if tmin > tmax:
+            tmin, tmax = tmax, tmin
         tensor = flat.histc(bins=self._num_bins, min=tmin, max=tmax)
         tensor = tensor.cpu().clone().detach()
         bins = torch.linspace(tmin, tmax, steps=self._num_bins + 1)
